/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::ArMemMemoryViewerWidgetController
 * \author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * \date       2020
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "ArMemMemoryViewerWidgetController.h"

#include <string>

#include "MemoryTreeWidgetBuilder.h"


namespace armarx
{
    QString ArMemMemoryViewerWidgetController::GetWidgetName()
    {
        return "MemoryX.ArMemMemoryViewer";
    }


    ArMemMemoryViewerWidgetController::ArMemMemoryViewerWidgetController()
    {
        widget.setupUi(getWidget());

        using This = ArMemMemoryViewerWidgetController;

        connect(widget.updateButton, &QPushButton::pressed, this, &This::update);
    }

    ArMemMemoryViewerWidgetController::~ArMemMemoryViewerWidgetController()
    {
    }


    void ArMemMemoryViewerWidgetController::loadSettings(QSettings* settings)
    {
        (void) settings;
    }
    void ArMemMemoryViewerWidgetController::saveSettings(QSettings* settings)
    {
        (void) settings;
    }


    QPointer<QDialog> ArMemMemoryViewerWidgetController::getConfigDialog(QWidget* parent)
    {
        ARMARX_IMPORTANT << "getConfigDialog() ";
        if (!configDialog)
        {
            configDialog = new SimpleConfigDialog(parent);
            configDialog->addProxyFinder<armem::MemoryInterfacePrx>({"VisionMemoryName", "Vision Memory", "*VisionMemory"});
        }
        return qobject_cast<QDialog*>(configDialog);
    }

    void ArMemMemoryViewerWidgetController::configured()
    {
        ARMARX_IMPORTANT << "configured() ";
        if (configDialog)
        {
            memoryName = configDialog->getProxyName("VisionMemoryName");
        }
    }


    void ArMemMemoryViewerWidgetController::onInitComponent()
    {
        if (memoryName.size() > 0)
        {
            usingProxy(memoryName);
        }
    }

    void ArMemMemoryViewerWidgetController::onConnectComponent()
    {
        if (memoryName.size() > 0)
        {
            getProxy(memory, memoryName);
            memoryReader = armem::MemoryReader(memory);
        }
    }


    void updateMemoryTree(armem::Memory& memory, QTreeWidget& tree)
    {
        armem::Time start = armem::Time::now();

        armem::MemoryTreeWidgetBuilder builder;
        builder.updateChildren(memory, &tree);

        ARMARX_INFO << "GUI update took " << (armem::Time::now() - start).toMilliSecondsDouble() << " ms.";
    }

    void ArMemMemoryViewerWidgetController::update()
    {
        if (!memoryReader)
        {
            return;
        }
        armem::Time start = armem::Time::now();
        armem::MemoryPtr queryResult = memoryReader.getEntityHistories();
        ARMARX_INFO << "Memory query took " << (armem::Time::now() - start).toMilliSecondsDouble() << " ms.";
        if (queryResult)
        {
            updateMemoryTree(*queryResult, *widget.memoryTree);
        }
        else
        {
            ARMARX_ERROR << "No query result";
        }
    }
}
