/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::ArMemMemoryViewerWidgetController
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <RobotAPI/gui-plugins/ArMemMemoryViewer/ui_ArMemMemoryViewerWidget.h>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/SimpleConfigDialog/SimpleConfigDialog.h>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <RobotAPI/interface/armem/MemoryInterface.h>
#include <RobotAPI/libraries/armem/client/MemoryReader.h>


namespace armarx
{
    /**
    \page RobotAPI-GuiPlugins-ArMemMemoryViewer ArMemMemoryViewer
    \brief The ArMemMemoryViewer allows visualizing ...

    \image html ArMemMemoryViewer.png
    The user can

    API Documentation \ref ArMemMemoryViewerWidgetController

    \see ArMemMemoryViewerGuiPlugin
    */


    /**
     * \class ArMemMemoryViewerGuiPlugin
     * \ingroup ArmarXGuiPlugins
     * \brief ArMemMemoryViewerGuiPlugin brief description
     *
     * Detailed description
     */

    /**
     * \class ArMemMemoryViewerWidgetController
     * \brief ArMemMemoryViewerWidgetController brief one line description
     *
     * Detailed description
     */
    class ARMARXCOMPONENT_IMPORT_EXPORT
        ArMemMemoryViewerWidgetController:
        public armarx::ArmarXComponentWidgetControllerTemplate < ArMemMemoryViewerWidgetController >
    {
        Q_OBJECT

    public:
        /// Returns the Widget name displayed in the ArmarXGui to create an instance of this class.
        static QString GetWidgetName();


        /// Controller Constructor
        explicit ArMemMemoryViewerWidgetController();
        /// Controller destructor
        virtual ~ArMemMemoryViewerWidgetController() override;

        void loadSettings(QSettings* settings) override;
        void saveSettings(QSettings* settings) override;

        QPointer<QDialog> getConfigDialog(QWidget* parent) override;
        void configured() override;

        void onInitComponent() override;
        void onConnectComponent() override;


    public slots:
        /* QT slot declarations */

        void update();


    signals:
        /* QT signal declarations */

    private:

        /// Widget Form
        Ui::ArMemMemoryViewerWidget widget;

        QPointer<SimpleConfigDialog> configDialog;

        std::string memoryName;
        armem::MemoryInterfacePrx memory;
        armem::MemoryReader memoryReader;

    };
}


