#pragma once

#include <functional>
#include <map>
#include <sstream>

#include <QTreeWidget>


#include <RobotAPI/libraries/armem/memory/Memory.h>


namespace armarx::armem
{

    /**
     * A class to efficiently build and maintain sorted items of `QTreeWidget`
     * or `QTreeWidgetItem` based on a sorted container matching the intended structure.
     */
    struct MemoryTreeWidgetBuilder
    {

        void updateChildren(const armem::Memory& memory, QTreeWidget* tree);
        void updateChildren(const std::map<std::string, const armem::Memory*>& memories, QTreeWidget* tree);

        void updateChildren(const armem::Memory& memory, QTreeWidgetItem* memoryItem);
        void updateChildren(const armem::CoreSegment& coreSeg, QTreeWidgetItem* coreSegItem);
        void updateChildren(const armem::ProviderSegment& provSeg, QTreeWidgetItem* provSegItem);
        void updateChildren(const armem::Entity& entity, QTreeWidgetItem* entityItem);
        void updateChildren(const armem::EntitySnapshot& snapshot, QTreeWidgetItem* snapshotItem);
        void updateChildren(const armem::EntityData& data, QTreeWidgetItem* parent);

    };

}
