#pragma once

// AronMacros
/************************************
 * TYPES ****************************
 ***********************************/

#define HANDLE_CONTAINER_TYPES \
    RUN_ARON_MACRO(Object, object, OBJECT) \
    RUN_ARON_MACRO(List, list, LIST) \
    RUN_ARON_MACRO(Dict, dict, DICT) \
    RUN_ARON_MACRO(Tuple, tuple, TUPLE) \

#define HANDLE_COMPLEX_TYPES \
    RUN_ARON_MACRO(EigenMatrix, eigenmatrix, EIGEN_MATRIX) \
    RUN_ARON_MACRO(IVTCByteImage, ivtcbyteimage, IVT_CBYTE_IMAGE) \
    RUN_ARON_MACRO(OpenCVMat, opencvmat, OPENCV_MAT) \
    RUN_ARON_MACRO(PCLPointcloud, pclpointcloud, PCL_POINTCLOUD)

#define HANDLE_PRIMITIVE_TYPES \
    RUN_ARON_MACRO(Int, int, INT) \
    RUN_ARON_MACRO(Long, long, LONG) \
    RUN_ARON_MACRO(Float, float, FLOAT) \
    RUN_ARON_MACRO(Double, double, DOUBLE) \
    RUN_ARON_MACRO(String, string, STRING) \
    RUN_ARON_MACRO(Bool, bool, BOOL)

#define HANDLE_ALL_ARON_TYPES \
    HANDLE_CONTAINER_TYPES \
    HANDLE_COMPLEX_TYPES \
    HANDLE_PRIMITIVE_TYPES

/************************************
 * DATA *****************************
 ***********************************/

#define HANDLE_CONTAINER_DATA \
    RUN_ARON_MACRO(List, list, LIST) \
    RUN_ARON_MACRO(Dict, dict, DICT)

#define HANDLE_COMPLEX_DATA \
    RUN_ARON_MACRO(NDArray, ndarray, NDARRAY)

#define HANDLE_ALL_ARON_DATA \
    HANDLE_CONTAINER_DATA \
    HANDLE_COMPLEX_DATA \
    HANDLE_PRIMITIVE_TYPES


module armarx
{
    module aron
    {

        /*************************
         * General Definitions ***
         ************************/
        sequence<byte> AronByteSequence;
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
        sequence<lowerType> Aron##upperType##Sequence;

        HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO


        /*************************
         * Aron Types ************
         ************************/
        module type
        {
            class AronType { };
            sequence<AronType> AronTypeList;
            dictionary<string, AronType> AronTypeDict;

            // Container Types (serialize to object/list)
            // Please note that either elementTypes xor acceptedType is set!!!
            class AronDictSerializerType extends AronType { AronTypeDict elementTypes; AronType acceptedType;};
            class AronListSerializerType extends AronType { AronTypeList elementTypes; AronType acceptedType;};

            class AronListType extends AronListSerializerType { };
            class AronTupleType extends AronListSerializerType { };

            class AronObjectType extends AronDictSerializerType { AronObjectType parent; string objectName; };
            class AronDictType extends AronDictSerializerType { };

            // Complex Types (serialize to ndarray)
            class AronNDArraySerializerType extends AronType { AronIntSequence dimensions; string typeName; };
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType##Type extends AronNDArraySerializerType { };

                    HANDLE_COMPLEX_TYPES
            #undef RUN_ARON_MACRO

            // Primitive Types
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType##Type extends AronType { };

                    HANDLE_PRIMITIVE_TYPES
            #undef RUN_ARON_MACRO
                };


        /*************************
         * Aron Data *************
         ************************/
        module data
        {
            class AronData { };
            sequence<AronData> AronDataList;
            dictionary<string, AronData> AronDataDict;

            // Container Data
            class AronList extends AronData { AronDataList elements; };
            class AronDict extends AronData { AronDataDict elements; };

            // Complex Data
            class AronNDArray extends AronData { AronIntSequence dimensions; AronByteSequence data; };

            // Basic Data
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType extends AronData { lowerType value; };

                    HANDLE_PRIMITIVE_TYPES
            #undef RUN_ARON_MACRO
                };

    };
};
