#pragma once

#include <sstream>
#include <stack>

#include <ArmarXGui/libraries/RemoteGui/Client/Widgets.h>

#include <RobotAPI/libraries/aron/aroncore/navigators/visitors/AronDataVisitor.h>


namespace armarx::armem
{



    struct RemoteGuiAronDataVisitor : public aron::visitor::AronDataVisitor
    {
        using GroupBox = armarx::RemoteGui::Client::GroupBox;
        using Label = armarx::RemoteGui::Client::Label;

        std::stack<GroupBox> groups;
        GroupBox result;


        virtual ~RemoteGuiAronDataVisitor() override;


        bool visitEnter(AronDictDataNavigator&) override
        {
            return visitEnter();
        }
        bool visitExit(AronDictDataNavigator&) override
        {
            return visitExit();
        }

        bool visitEnter(AronListDataNavigator& list) override
        {
            (void) list;
            // ARMARX_INFO << "- List Enter (path: '" << list.getPath().toString() << "')";
            return visitEnter();
        }
        bool visitExit(AronListDataNavigator& list) override
        {
            (void) list;
            // ARMARX_INFO << "- List Exit (path: '" << list.getPath().toString() << "')";
            return visitExit();
        }

        // Same for Dict and List
        bool visitEnter();
        bool visitExit();


        bool visit(AronBoolDataNavigator& b) override
        {
            this->addValueLabel(b, "bool");
            return true;
        }
        bool visit(AronDoubleDataNavigator& d) override
        {
            this->addValueLabel(d, "double");
            return true;
        }
        bool visit(AronFloatDataNavigator& f) override
        {
            this->addValueLabel(f, "float");
            return true;
        }
        bool visit(AronIntDataNavigator& i) override
        {
            this->addValueLabel(i, "int");
            return true;
        }
        bool visit(AronLongDataNavigator& l) override
        {
            this->addValueLabel(l, "long");
            return true;
        }
        bool visit(AronStringDataNavigator& string) override
        {
            this->addValueLabel(string, "string");
            return true;
        }

        bool visit(AronNDArrayDataNavigator& array) override
        {
            (void) array;
            // ARMARX_INFO << "- NdArray (path: '" << array.getPath().toString() << "')";
            return true;
        }


    private:

        template <class Navigator>
        void addValueLabel(Navigator& n, const std::string& typeName)
        {
            groups.top().addChild(Label(this->makeValueLabelText(n, typeName)));
        }
        template <class Navigator>
        std::string makeValueLabelText(Navigator& n, const std::string& typeName)
        {
            std::stringstream ss;
            ss << n.getPath().getLastElement() << ": " << typeName << " = ";
            streamValueText(n, ss);
            return ss.str();
        }

        template <class Navigator>
        void streamValueText(Navigator& n, std::stringstream& ss)
        {
            ss << n.getValue();
        }
        void streamValueText(AronStringDataNavigator& n, std::stringstream& ss);
        void streamValueText(AronNDArrayDataNavigator& n, std::stringstream& ss);

    };


}
