#pragma once

#include <map>
#include <memory>
#include <string>

#include "ProviderSegment.h"
#include "detail/EntityStorage.h"


namespace armarx::armem
{

    class CoreSegment;
    using CoreSegmentPtr = std::unique_ptr<CoreSegment>;

    /**
     * @brief Data of a core segment containing multiple provider segments.
     */
    class CoreSegment : public detail::TypedEntityStorage<ProviderSegmentPtr>
    {
    public:

        using TypedEntityStorage::TypedEntityStorage;

        CoreSegment(const CoreSegment& other);
        CoreSegment& operator=(const CoreSegment& other);

        ~CoreSegment() override;


        bool hasProviderSegment(const std::string& name) const;

        ProviderSegment& getProviderSegment(const std::string& name);
        const ProviderSegment& getProviderSegment(const std::string& name) const;

        using TypedEntityStorage::getEntity;
        const Entity& getEntity(const MemoryID& id) const override;


        using TypedEntityStorage::update;
        virtual MemoryID update(const InternalEntityUpdate& update) override;

        /**
         * @brief Add an empty provider segment with the given name and optional provider segment type.
         * @param name The segment name.
         * @param providerSegmentType The provider type. If nullptr, the core segment type is used.
         * @return The added provider segment.
         */
        ProviderSegment& addProviderSegment(const std::string& name, aron::type::AronTypePtr providerSegmentType = nullptr);
        /// Copy and insert a provider segment.
        ProviderSegment& addProviderSegment(const ProviderSegment& providerSegment);
        /// Move and insert a provider segment.
        ProviderSegment& addProviderSegment(ProviderSegmentPtr&& providerSegment);

        /// Get an cleared copy of `*this` (without provider segments).
        CoreSegmentPtr getEmptyCopy() const;


    public:

        /// The provider segments.
        std::map<std::string, ProviderSegmentPtr> providerSegments;


        // MemoryLevel interface
    public:
        std::string getLevelName() const override
        {
            return "core segment";
        }

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override
        {
            return providerSegments;
        }

    };

}
