#pragma once

#include <memory>

#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigator.h>

#include "../core/Time.h"

#include "InternalCommit.h"
#include "detail/MemoryLevel.h"


namespace armarx::armem
{

    /**
     * @brief Metadata of an entity instance.
     */
    struct EntityMetadata
    {
        /// Time when this value was created.
        Time timeCreated;
        /// Time when this value was sent to the memory.
        Time timeSent;
        /// Time when this value has arrived at the memory.
        Time timeArrived;

        /// An optional confidence, may be used for things like decay.
        float confidence = 1.0;
    };


    /**
     * @brief Data of a single entity instance.
     */
    struct EntityData : public detail::MemoryLevel
    {
        int index;

        /// The actual data.
        aron::data::AronDataPtr data;
        /// Allows investigating `data`'s structure. May be nullptr.
        aron::datanavigator::AronDataNavigatorPtr dataNavigator;

        EntityMetadata metadata;

        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        void update(const InternalEntityUpdate& update, int index);


        // MemoryLevel interface
    public:
        std::string getKeyString() const override
        {
            return std::to_string(index);
        }
        std::string getLevelName() const override
        {
            return "entity data";
        }

    };

    using EntityDataPtr = std::unique_ptr<EntityData>;

}
