#pragma once

#include <memory>
#include <vector>

#include "../core/Time.h"
#include "../core/MemoryID.h"

#include "EntityData.h"
#include "detail/MemoryLevel.h"
#include "detail/MemoryContainer.h"


namespace armarx::armem
{

    /**
     * @brief Data of an entity at one point in time.
     */
    class EntitySnapshot :
        public detail::MemoryContainerImpl<std::vector<EntityDataPtr>>
    {
    public:

        EntitySnapshot() = default;
        EntitySnapshot(Time time);

        /// Copy the instances from `other` to this.
        EntitySnapshot(const EntitySnapshot& other);
        /// Copy the instances from `other` to this.
        EntitySnapshot& operator=(const EntitySnapshot& other);


        void update(const InternalEntityUpdate& update);


        /**
         * @brief Get the given instance.
         * @param index The instance's index.
         * @return The instance.
         * @throw `armem::error::MissingEntry` If the given index is invalid.
         */
        EntityData& getInstance(int index);
        /**
         * @brief Get the given instance.
         * @param index The instance's index.
         * @return The instance.
         * @throw `armem::error::MissingEntry` If the given index is invalid.
         * @throw `armem::error::InvalidMemoryID` If memory ID does not have an instance index.
         */
        EntityData& getInstance(MemoryID id);


    public:

        Time time = Time::microSeconds(-1);
        std::vector<EntityDataPtr> instances;



        // MemoryLevel interface
    public:
        std::string getKeyString() const override
        {
            return toDateTimeMilliSeconds(time);
        }
        std::string getLevelName() const override
        {
            return "entity snapshot";
        }

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override
        {
            return instances;
        }

    };

    using EntitySnapshotPtr = std::unique_ptr<EntitySnapshot>;


}
