#pragma once

#include <string>

#include "MemoryLevel.h"


namespace armarx::armem::detail
{

    /**
     * @brief A memory container at some point in the hierarchy.
     *
     * This class containes some convenience functions to allow more uniform
     * access to the different memory levels.
     */
    class MemoryContainer : public MemoryLevel
    {
    public:

        virtual ~MemoryContainer();

        // Container methods

        virtual bool empty() const = 0;
        virtual std::size_t size() const = 0;
        virtual void clear() = 0;

    };



    /**
     * @class Provides default implmentations of `MemoryContainer`, as well as
     * iterators (which requires a template).
     */
    template <class _ContainerT>
    class MemoryContainerImpl : public MemoryContainer
    {
    public:
        using ContainerT = _ContainerT;

    public:

        virtual ~MemoryContainerImpl()
        {}


        // Container methods

        virtual bool empty() const
        {
            return _underlyingContainer().empty();
        }
        virtual std::size_t size() const
        {
            return _underlyingContainer().size();
        }
        virtual void clear()
        {
            return _underlyingContainer().clear();
        }

        auto begin() const
        {
            return _underlyingContainer().begin();
        }
        auto begin()
        {
            return _underlyingContainer().begin();
        }
        auto end() const
        {
            return _underlyingContainer().end();
        }
        auto end()
        {
            return _underlyingContainer().end();
        }


    protected:

        virtual ContainerT& _underlyingContainer() = 0;
        virtual const ContainerT& _underlyingContainer() const
        {
            return const_cast<const ContainerT&>(const_cast<MemoryContainerImpl*>(this)->_underlyingContainer());
        }

    };

}
