#pragma once

#include <RobotAPI/interface/armem/query.h>

#include "../memory/Memory.h"


namespace armarx::armem
{

    /**
     * @brief Base class for memory query processors.
     */
    template <class DataT, class QueryT>
    class BaseQueryProcessor
    {
    public:

        using DataPtrT = std::unique_ptr<DataT>;
        using QueryPtrT = ::IceInternal::Handle<QueryT>;
        using QuerySeqT = std::vector<QueryPtrT>;


    public:


        virtual ~BaseQueryProcessor()
        {}


        DataPtrT process(const QueryT& query, const DataT& data) const
        {
            DataPtrT result = data.getEmptyCopy();
            this->process(*result, query, data);
            return result;
        }
        DataPtrT process(const QueryPtrT& query, const DataPtrT& data) const
        {
            return this->process(*query, *data);
        }

        DataPtrT process(const QuerySeqT& queries, const DataT& data) const
        {
            DataPtrT result = data.getEmptyCopy();
            this->process(*result, queries, data);
            return result;
        }


        void process(DataT& result, const QuerySeqT& queries, const DataT& data) const
        {
            for (const auto& query : queries)
            {
                this->process(result, *query, data);
            }
        }

        virtual void process(DataT& result, const QueryT& query, const DataT& data) const = 0;

    };

}
