#include "AronDataVisitor.h"

#include <SimoxUtility/meta/type_name.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::aron::visitor
{


    AronDataVisitor::AronDataVisitor()
    {
    }

    AronDataVisitor::~AronDataVisitor()
    {
    }


    bool AronDataVisitor::applyTo(data::AronDataPtr& data)
    {
        ARMARX_CHECK_NOT_NULL(data);
        return applyTo(*AronDataNavigator::FromAronData(data));
    }


    bool AronDataVisitor::applyTo(AronDataNavigator& navigator)
    {
        auto handleCollection = [this](auto & n)
        {
            if (!visitEnter(n))
            {
                return false;
            }
            if (!applyToChildren(n))
            {
                return false;
            }
            if (!visitExit(n))
            {
                return false;
            }
            return true;
        };

        if (auto n = dynamic_cast<AronDictDataNavigator*>(&navigator))
        {
            return handleCollection(*n);
        }
        else if (auto n = dynamic_cast<AronListDataNavigator*>(&navigator))
        {
            return handleCollection(*n);
        }

        ARMARX_CHECK_EQUAL(navigator.childrenSize(), 0)
                << simox::meta::get_type_name(navigator);

        if (auto n = dynamic_cast<AronBoolDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        else if (auto n = dynamic_cast<AronDoubleDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        else if (auto n = dynamic_cast<AronFloatDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        else if (auto n = dynamic_cast<AronIntDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        else if (auto n = dynamic_cast<AronLongDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        else if (auto n = dynamic_cast<AronNDArrayDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        else if (auto n = dynamic_cast<AronStringDataNavigator*>(&navigator))
        {
            return visit(*n);
        }
        ARMARX_CHECK(false) << "Unhandled AronDataNavigatorType '" << simox::meta::get_type_name(navigator) << "'.";
    }


    bool AronDataVisitor::applyToChildren(AronDataNavigator& navigator)
    {
        for (auto& child : navigator.getChildren())
        {
            if (child)
            {
                if (!applyTo(*child))
                {
                    return false;
                }
            }
        }
        return true;
    }

}
